@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP #-}
 module Options.Applicative.Builder (
   -- * Parser builders
   --
@@ -38,19 +37,16 @@ module Options.Applicative.Builder (
   showDefaultWith,
   showDefault,
   metavar,
-  eitherReader,
   noArgError,
   ParseError(..),
   hidden,
   internal,
   command,
+  commandGroup,
   completeWith,
   action,
   completer,
   idm,
-#if __GLASGOW_HASKELL__ > 702
-  (<>),
-#endif
   mappend,
 
   -- * Readers
@@ -58,6 +54,8 @@ module Options.Applicative.Builder (
   -- | A collection of basic 'Option' readers.
   auto,
   str,
+  maybeReader,
+  eitherReader,
   disabled,
   readerAbort,
   readerError,
@@ -81,6 +79,7 @@ module Options.Applicative.Builder (
   multiSuffix,
   disambiguate,
   showHelpOnError,
+  showHelpOnEmpty,
   noBacktrack,
   columns,
   prefs,
@@ -95,12 +94,8 @@ module Options.Applicative.Builder (
   CommandFields
   ) where
 
-import Control.Applicative (pure, (<|>))
-import Data.Monoid (Monoid (..)
-#if __GLASGOW_HASKELL__ > 702
-  , (<>)
-#endif
-  )
+import Control.Applicative
+import Data.Semigroup hiding (option)
 
 import Options.Applicative.Builder.Completer
 import Options.Applicative.Builder.Internal
@@ -121,6 +116,15 @@ auto = eitherReader $ \arg -> case reads arg of
 str :: ReadM String
 str = readerAsk
 
+-- | Convert a function in the 'Either' monad to a reader.
+eitherReader :: (String -> Either String a) -> ReadM a
+eitherReader f = readerAsk >>= either readerError return . f
+
+-- | Convert a function in the 'Maybe' monad to a reader.
+maybeReader :: (String -> Maybe a) -> ReadM a
+maybeReader f = eitherReader $ \arg ->
+  maybe (Left $ "cannot parse value `" ++ arg ++ "'") pure . f $ arg
+
 -- | Null 'Option' reader. All arguments will fail validation.
 disabled :: ReadM a
 disabled = readerError "disabled option"
@@ -141,6 +145,9 @@ long = fieldMod . name . OptLong
 -- do not use it with combinators such as 'some' or 'many', as
 -- these combinators continue until a failure occurs.
 -- Careless use will thus result in a hang.
+--
+-- To display the default value, combine with showDefault or
+-- showDefaultWith.
 value :: HasValue f => a -> Mod f a
 value x = Mod id (DefaultProp (Just x) Nothing) id
 
@@ -161,10 +168,6 @@ help s = optionMod $ \p -> p { propHelp = paragraph s }
 helpDoc :: Maybe Doc -> Mod f a
 helpDoc doc = optionMod $ \p -> p { propHelp = Chunk doc }
 
--- | Convert a function in the 'Either' monad to a reader.
-eitherReader :: (String -> Either String a) -> ReadM a
-eitherReader f = readerAsk >>= either readerError return . f
-
 -- | Specify the error to display when no argument is provided to this option.
 noArgError :: ParseError -> Mod OptionFields a
 noArgError e = fieldMod $ \p -> p { optNoArgError = e }
@@ -186,6 +189,11 @@ command :: String -> ParserInfo a -> Mod CommandFields a
 command cmd pinfo = fieldMod $ \p ->
   p { cmdCommands = (cmd, pinfo) : cmdCommands p }
 
+-- | Add a description to a group of commands.
+commandGroup :: String -> Mod CommandFields a
+commandGroup g = fieldMod $ \p ->
+  p { cmdGroup = Just g }
+
 -- | Add a list of possible completion values.
 completeWith :: HasCompleter f => [String] -> Mod f a
 completeWith xs = completer (listCompleter xs)
@@ -212,7 +220,8 @@ subparser :: Mod CommandFields a -> Parser a
 subparser m = mkParser d g rdr
   where
     Mod _ d g = metavar "COMMAND" `mappend` m
-    rdr = uncurry CmdReader (mkCommand m)
+    (groupName, cmds, subs) = mkCommand m
+    rdr = CmdReader groupName cmds subs
 
 -- | Builder for an argument parser.
 argument :: ReadM a -> Mod ArgumentFields a -> Parser a
@@ -229,6 +238,10 @@ strArgument = argument str
 --
 -- A flag that switches from a \"default value\" to an \"active value\" when
 -- encountered. For a simple boolean value, use `switch` instead.
+--
+-- /Note/: Because this parser will never fail, it can not be used with
+-- combinators such as 'some' or 'many', as these combinators continue until
+-- a failure occurs. See @flag'@.
 flag :: a                         -- ^ default value
      -> a                         -- ^ active value
      -> Mod FlagFields a          -- ^ option modifier
@@ -244,7 +257,12 @@ flag defv actv m = flag' actv m <|> pure defv
 --
 -- > length <$> many (flag' () (short 't'))
 --
--- is a parser that counts the number of "-t" arguments on the command line.
+-- is a parser that counts the number of "-t" arguments on the command line,
+-- alternatively
+--
+-- > flag' True (long "on") <|> flag' False (long "off")
+--
+-- will require the user to enter '--on' or '--off' on the command line.
 flag' :: a                         -- ^ active value
       -> Mod FlagFields a          -- ^ option modifier
       -> Parser a
@@ -256,6 +274,10 @@ flag' actv (Mod f d g) = mkParser d g rdr
 
 -- | Builder for a boolean flag.
 --
+-- /Note/: Because this parser will never fail, it can not be used with
+-- combinators such as 'some' or 'many', as these combinators continue until
+-- a failure occurs. See @flag'@.
+--
 -- > switch = flag False True
 switch :: Mod FlagFields Bool -> Parser Bool
 switch = flag False True
@@ -285,6 +307,12 @@ nullOption :: ReadM a -> Mod OptionFields a -> Parser a
 nullOption = option
 
 -- | Builder for an option using the given reader.
+--
+-- This is a regular option, and should always have either a @long@ or
+-- @short@ name specified in the modifiers (or both).
+--
+-- > nameParser = option str ( long "name" <> short 'n' )
+--
 option :: ReadM a -> Mod OptionFields a -> Parser a
 option r m = mkParser d g rdr
   where
@@ -299,7 +327,10 @@ newtype InfoMod a = InfoMod
 
 instance Monoid (InfoMod a) where
   mempty = InfoMod id
-  mappend m1 m2 = InfoMod $ applyInfoMod m2 . applyInfoMod m1
+  mappend = (<>)
+
+instance Semigroup (InfoMod a) where
+  m1 <> m2 = InfoMod $ applyInfoMod m2 . applyInfoMod m1
 
 -- | Show a full description in the help text of this parser.
 fullDesc :: InfoMod a
@@ -362,7 +393,10 @@ newtype PrefsMod = PrefsMod
 
 instance Monoid PrefsMod where
   mempty = PrefsMod id
-  mappend m1 m2 = PrefsMod $ applyPrefsMod m2 . applyPrefsMod m1
+  mappend = (<>)
+
+instance Semigroup PrefsMod where
+  m1 <> m2 = PrefsMod $ applyPrefsMod m2 . applyPrefsMod m1
 
 multiSuffix :: String -> PrefsMod
 multiSuffix s = PrefsMod $ \p -> p { prefMultiSuffix = s }
@@ -373,6 +407,9 @@ disambiguate = PrefsMod $ \p -> p { prefDisambiguate = True }
 showHelpOnError :: PrefsMod
 showHelpOnError = PrefsMod $ \p -> p { prefShowHelpOnError = True }
 
+showHelpOnEmpty :: PrefsMod
+showHelpOnEmpty = PrefsMod $ \p -> p { prefShowHelpOnEmpty = True }
+
 noBacktrack :: PrefsMod
 noBacktrack = PrefsMod $ \p -> p { prefBacktrack = False }
 
@@ -386,6 +423,7 @@ prefs m = applyPrefsMod m base
       { prefMultiSuffix = ""
       , prefDisambiguate = False
       , prefShowHelpOnError = False
+      , prefShowHelpOnEmpty = False
       , prefBacktrack = True
       , prefColumns = 80 }
 
