@@ -3,8 +3,8 @@ module Options.Applicative.Builder.Internal (
   Mod(..),
   HasName(..),
   HasCompleter(..),
-  HasValue,
-  HasMetavar,
+  HasValue(..),
+  HasMetavar(..),
   OptionFields(..),
   FlagFields(..),
   CommandFields(..),
@@ -23,9 +23,10 @@ module Options.Applicative.Builder.Internal (
   internal
   ) where
 
-import Control.Applicative (pure, (<*>), empty, (<|>))
+import Control.Applicative
 import Control.Monad (mplus)
-import Data.Monoid (Monoid(..))
+import Data.Semigroup hiding (Option)
+import Prelude
 
 import Options.Applicative.Common
 import Options.Applicative.Types
@@ -40,7 +41,8 @@ data FlagFields a = FlagFields
   , flagActive :: a }
 
 data CommandFields a = CommandFields
-  { cmdCommands :: [(String, ParserInfo a)] }
+  { cmdCommands :: [(String, ParserInfo a)]
+  , cmdGroup :: Maybe String }
 
 data ArgumentFields a = ArgumentFields
   { argCompleter :: Completer }
@@ -88,7 +90,10 @@ data DefaultProp a = DefaultProp
 
 instance Monoid (DefaultProp a) where
   mempty = DefaultProp Nothing Nothing
-  mappend (DefaultProp d1 s1) (DefaultProp d2 s2) =
+  mappend = (<>)
+
+instance Semigroup (DefaultProp a) where
+  (DefaultProp d1 s1) <> (DefaultProp d2 s2) =
     DefaultProp (d1 `mplus` d2) (s1 `mplus` s2)
 
 -- | An option modifier.
@@ -128,8 +133,11 @@ fieldMod f = Mod f mempty id
 
 instance Monoid (Mod f a) where
   mempty = Mod id mempty id
-  Mod f1 d1 g1 `mappend` Mod f2 d2 g2
-    = Mod (f2 . f1) (d2 `mappend` d1) (g2 . g1)
+  mappend = (<>)
+
+instance Semigroup (Mod f a) where
+  Mod f1 d1 g1 <> Mod f2 d2 g2
+    = Mod (f2 . f1) (d2 <> d1) (g2 . g1)
 
 -- | Base default properties.
 baseProps :: OptProperties
@@ -139,11 +147,11 @@ baseProps = OptProperties
   , propHelp = mempty
   , propShowDefault = Nothing }
 
-mkCommand :: Mod CommandFields a -> ([String], String -> Maybe (ParserInfo a))
-mkCommand m = (map fst cmds, (`lookup` cmds))
+mkCommand :: Mod CommandFields a -> (Maybe String, [String], String -> Maybe (ParserInfo a))
+mkCommand m = (group, map fst cmds, (`lookup` cmds))
   where
     Mod f _ _ = m
-    CommandFields cmds = f (CommandFields [])
+    CommandFields cmds group = f (CommandFields [] Nothing)
 
 mkParser :: DefaultProp a
          -> (OptProperties -> OptProperties)
