@@ -21,8 +21,9 @@ module Options.Applicative.Extra (
   CompletionResult(..),
   ) where
 
-import Control.Applicative (pure, (<$>), (<|>), (<**>))
-import Data.Monoid (mempty, mconcat)
+import Control.Applicative
+import Data.Monoid
+import Prelude
 import System.Environment (getArgs, getProgName)
 import System.Exit (exitSuccess, exitWith, ExitCode(..))
 import System.IO (hPutStrLn, stderr)
@@ -50,9 +51,9 @@ helper = abortOption ShowHelpText $ mconcat
 hsubparser :: Mod CommandFields a -> Parser a
 hsubparser m = mkParser d g rdr
   where
-    Mod _ d g = m `mappend` metavar "COMMAND"
-    (cmds, subs) = mkCommand m
-    rdr = CmdReader cmds (fmap add_helper . subs)
+    Mod _ d g = metavar "COMMAND" `mappend` m
+    (groupName, cmds, subs) = mkCommand m
+    rdr = CmdReader groupName cmds (fmap add_helper . subs)
     add_helper pinfo = pinfo
       { infoParser = infoParser pinfo <**> helper }
 
@@ -146,11 +147,11 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
   in (h, exit_code, prefColumns pprefs)
   where
     exit_code = case msg of
-      ErrorMsg _     -> ExitFailure (infoFailureCode pinfo)
-      UnknownError   -> ExitFailure (infoFailureCode pinfo)
-      MissingError _ -> ExitFailure (infoFailureCode pinfo)
-      ShowHelpText   -> ExitSuccess
-      InfoMsg  _     -> ExitSuccess
+      ErrorMsg _       -> ExitFailure (infoFailureCode pinfo)
+      UnknownError     -> ExitFailure (infoFailureCode pinfo)
+      MissingError _ _ -> ExitFailure (infoFailureCode pinfo)
+      ShowHelpText     -> ExitSuccess
+      InfoMsg  _       -> ExitSuccess
 
     with_context :: [Context]
                  -> ParserInfo a
@@ -166,11 +167,13 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
         , fmap (indent 2) . infoProgDesc $ i ]
 
     error_help = errorHelp $ case msg of
-      ShowHelpText   -> mempty
-      ErrorMsg m     -> stringChunk m
-      InfoMsg  m     -> stringChunk m
-      MissingError x -> stringChunk "Missing:" <<+>> fold_tree x
-      UnknownError   -> mempty
+      ShowHelpText                  -> mempty
+      ErrorMsg m                    -> stringChunk m
+      InfoMsg  m                    -> stringChunk m
+      MissingError CmdStart _        | prefShowHelpOnEmpty pprefs
+                                    -> mempty
+      MissingError _ (SomeParser x) -> stringChunk "Missing:" <<+>> missingDesc pprefs x
+      UnknownError                  -> mempty
 
     base_help :: ParserInfo a -> ParserHelp
     base_help i
@@ -183,8 +186,10 @@ parserFailure pprefs pinfo msg ctx = ParserFailure $ \progn ->
         f = footerHelp (infoFooter i)
 
     show_full_help = case msg of
-      ShowHelpText -> True
-      _            -> prefShowHelpOnError pprefs
+      ShowHelpText             -> True
+      MissingError CmdStart  _ | prefShowHelpOnEmpty pprefs
+                               -> True
+      _                        -> prefShowHelpOnError pprefs
 
 renderFailure :: ParserFailure ParserHelp -> String -> (String, ExitCode)
 renderFailure failure progn =
