@@ -1,7 +1,6 @@
 {-# LANGUAGE ExistentialQuantification #-}
 module Options.Applicative.Internal
   ( P
-  , Context(..)
   , MonadP(..)
   , ParseError(..)
 
@@ -15,8 +14,6 @@ module Options.Applicative.Internal
 
   , Completion
   , runCompletion
-  , SomeParser(..)
-  , ComplError(..)
   , contextNames
 
   , ListT
@@ -29,7 +26,8 @@ module Options.Applicative.Internal
   , disamb
   ) where
 
-import Control.Applicative (Applicative(..), Alternative(..), (<$>))
+import Control.Applicative
+import Prelude
 import Control.Monad (MonadPlus(..), liftM, ap, guard)
 import Control.Monad.Trans.Class (MonadTrans, lift)
 import Control.Monad.Trans.Except
@@ -48,7 +46,7 @@ class (Alternative m, MonadPlus m) => MonadP m where
   missingArgP :: ParseError -> Completer -> m a
   tryP :: m a -> m (Either ParseError a)
   errorP :: ParseError -> m a
-  exitP :: Parser b -> Either ParseError a -> m a
+  exitP :: IsCmdStart -> Parser b -> Maybe a -> m a
 
 newtype P a = P (ExceptT ParseError (StateT [Context] (Reader ParserPrefs)) a)
 
@@ -71,10 +69,6 @@ instance MonadPlus P where
   mzero = P mzero
   mplus (P x) (P y) = P $ mplus x y
 
-
-data Context
-  = forall a . Context String (ParserInfo a)
-
 contextNames :: [Context] -> [String]
 contextNames ns =
   let go (Context n _) = n
@@ -87,7 +81,7 @@ instance MonadP P where
 
   missingArgP e _ = errorP e
   tryP (P p) = P $ lift $ runExceptT p
-  exitP _ = P . (either throwE return)
+  exitP i p = P . maybe (throwE . MissingError i . SomeParser $ p) return
   errorP = P . throwE
 
 hoistMaybe :: MonadPlus m => Maybe a -> m a
@@ -112,12 +106,6 @@ withReadM f = ReadM . mapReaderT (withExcept f') . unReadM
     f' (ErrorMsg err) = ErrorMsg (f err)
     f' e = e
 
-data SomeParser = forall a . SomeParser (Parser a)
-
-data ComplError
-  = ComplParseError String
-  | ComplExit
-
 data ComplResult a
   = ComplParser SomeParser
   | ComplOption Completer
@@ -166,7 +154,7 @@ instance MonadP Completion where
 
   missingArgP _ = Completion . lift . lift . ComplOption
   tryP (Completion p) = Completion $ catchE (Right <$> p) (return . Left)
-  exitP p _ = Completion . lift . lift . ComplParser $ SomeParser p
+  exitP _ p _ = Completion . lift . lift . ComplParser $ SomeParser p
   errorP = Completion . throwE
 
 runCompletion :: Completion r -> ParserPrefs -> Maybe (Either SomeParser Completer)
