@@ -27,6 +27,9 @@ module Options.Applicative.Types (
   OptHelpInfo(..),
   OptTree(..),
   ParserHelp(..),
+  SomeParser(..),
+  Context(..),
+  IsCmdStart(..),
 
   fromM,
   oneM,
@@ -40,30 +43,37 @@ module Options.Applicative.Types (
   ) where
 
 import Control.Applicative
-  (Applicative(..), Alternative(..), (<$>), optional)
 import Control.Monad (ap, liftM, MonadPlus, mzero, mplus)
 import Control.Monad.Trans.Except (Except, throwE)
 import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.Reader (ReaderT, ask)
-import Data.Monoid (Monoid(..))
+import Data.Semigroup hiding (Option)
+import Prelude
+
 import System.Exit (ExitCode(..))
 
 import Options.Applicative.Help.Types
 import Options.Applicative.Help.Pretty
 import Options.Applicative.Help.Chunk
 
+
 data ParseError
   = ErrorMsg String
   | InfoMsg String
   | ShowHelpText
   | UnknownError
-  | MissingError (OptTree (Chunk Doc))
+  | MissingError IsCmdStart SomeParser
+
+data IsCmdStart = CmdStart | CmdCont
   deriving Show
 
 instance Monoid ParseError where
   mempty = UnknownError
-  mappend m UnknownError = m
-  mappend _ m = m
+  mappend = (<>)
+
+instance Semigroup ParseError where
+  m <> UnknownError = m
+  _ <> m = m
 
 -- | A full description for a runnable 'Parser' for a program.
 data ParserInfo a = ParserInfo
@@ -88,6 +98,8 @@ data ParserPrefs = ParserPrefs
                                  -- (default: False)
   , prefShowHelpOnError :: Bool  -- ^ always show help text on parse errors
                                  -- (default: False)
+  , prefShowHelpOnEmpty :: Bool  -- ^ show the help text for a command or subcommand
+                                 -- if it fails with no input (default: False)
   , prefBacktrack :: Bool        -- ^ backtrack to parent parser when a
                                  -- subcommand fails (default: True)
   , prefColumns :: Int           -- ^ number of columns in the terminal, used to
@@ -119,6 +131,12 @@ data Option a = Option
   , optProps :: OptProperties            -- ^ properties of this option
   }
 
+data SomeParser = forall a . SomeParser (Parser a)
+
+-- | Subparser context, containing the 'name' of the subparser, and its parser info.
+--   Used by parserFailure to display relevant usage information when parsing inside a subparser fails.
+data Context = forall a . Context String (ParserInfo a)
+
 instance Show (Option a) where
     show opt = "Option {optProps = " ++ show (optProps opt) ++ "}"
 
@@ -173,13 +191,14 @@ data OptReader a
   = OptReader [OptName] (CReader a) ParseError          -- ^ option reader
   | FlagReader [OptName] !a                             -- ^ flag reader
   | ArgReader (CReader a)                               -- ^ argument reader
-  | CmdReader [String] (String -> Maybe (ParserInfo a)) -- ^ command reader
+  | CmdReader (Maybe String)
+              [String] (String -> Maybe (ParserInfo a)) -- ^ command reader
 
 instance Functor OptReader where
   fmap f (OptReader ns cr e) = OptReader ns (fmap f cr) e
   fmap f (FlagReader ns x) = FlagReader ns (f x)
   fmap f (ArgReader cr) = ArgReader (fmap f cr)
-  fmap f (CmdReader cs g) = CmdReader cs ((fmap . fmap) f . g)
+  fmap f (CmdReader n cs g) = CmdReader n cs ((fmap . fmap) f . g)
 
 -- | A @Parser a@ is an option parser returning a value of type 'a'.
 data Parser a
@@ -298,9 +317,13 @@ instance Monad ParserResult where
 
 type Args = [String]
 
+-- | Policy for how to handle options within the parse
 data ArgPolicy
-  = SkipOpts
-  | AllowOpts
+  = SkipOpts  -- ^ Inputs beginning with `-` or `--` are treated
+              --   as options or flags, and can be mixed with arguments.
+  | AllowOpts -- ^ All input is treated as positional arguments.
+              --   Used after a bare `--` input, and also with
+              --   `noIntersperse` policy.
   deriving (Eq, Show)
 
 data OptHelpInfo = OptHelpInfo
